# Copyright (C) 2020 Esteban López Rodríguez <gnu_stallman@protonmail.ch>
#
# This file is part of Spike.
#
# Spike is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Spike is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from distutils.core import setup

setup(
    name="spike-telegram",
    version="0.1.0.1",
    packages=['spike', 'spike.tg', 'spike.common', 'spike.config', 'spike.tui'],
    license="GPL-3",
    author="Esteban López Rodríguez",
    author_email="gnu_stallman@protonmail.ch",
    url="https://gitlab.com/libre_hackerman/spike",
    description="Telegram client in Ncurses, written in Python",
    scripts=["bin/spike-telegram"],
    data_files=[("share/spike-telegram", ["resources/plup.wav"])],
    install_requires=["python-telegram", "emoji"]
)
